"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tracer = void 0;
const commons_1 = require("@aws-lambda-powertools/commons");
const config_1 = require("./config");
const provider_1 = require("./provider");
const aws_xray_sdk_core_1 = require("aws-xray-sdk-core");
class Tracer extends commons_1.Utility {
    constructor(options = {}) {
        super();
        this.captureError = true;
        this.captureHTTPsRequests = true;
        this.captureResponse = true;
        this.tracingEnabled = true;
        this.setOptions(options);
        this.provider = new provider_1.ProviderService();
        if (this.isTracingEnabled() && this.captureHTTPsRequests) {
            this.provider.captureHTTPsGlobal();
        }
        if (!this.isTracingEnabled()) {
            this.provider.setContextMissingStrategy(() => ({}));
        }
    }
    addErrorAsMetadata(error, remote) {
        if (!this.isTracingEnabled()) {
            return;
        }
        const subsegment = this.getSegment();
        if (subsegment === undefined) {
            return;
        }
        if (!this.captureError) {
            subsegment.addErrorFlag();
            return;
        }
        subsegment.addError(error, remote || false);
    }
    addResponseAsMetadata(data, methodName) {
        if (data === undefined ||
            !this.captureResponse ||
            !this.isTracingEnabled()) {
            return;
        }
        this.putMetadata(`${methodName} response`, data);
    }
    addServiceNameAnnotation() {
        if (!this.isTracingEnabled()) {
            return;
        }
        this.putAnnotation('Service', this.serviceName);
    }
    annotateColdStart() {
        if (this.isTracingEnabled()) {
            this.putAnnotation('ColdStart', this.getColdStart());
        }
    }
    captureAWS(aws) {
        if (!this.isTracingEnabled())
            return aws;
        return this.provider.captureAWS(aws);
    }
    captureAWSClient(service) {
        if (!this.isTracingEnabled())
            return service;
        try {
            return this.provider.captureAWSClient(service);
        }
        catch (error) {
            try {
                this.provider.captureAWSClient(service.service);
                return service;
            }
            catch {
                throw error;
            }
        }
    }
    captureAWSv3Client(service) {
        if (!this.isTracingEnabled())
            return service;
        return this.provider.captureAWSv3Client(service);
    }
    captureLambdaHandler(options) {
        return (_target, _propertyKey, descriptor) => {
            const originalMethod = descriptor.value;
            const tracerRef = this;
            descriptor.value = function (event, context, callback) {
                const handlerRef = this;
                if (!tracerRef.isTracingEnabled()) {
                    return originalMethod.apply(handlerRef, [event, context, callback]);
                }
                return tracerRef.provider.captureAsyncFunc(`## ${process.env._HANDLER}`, async (subsegment) => {
                    tracerRef.annotateColdStart();
                    tracerRef.addServiceNameAnnotation();
                    let result;
                    try {
                        result = await originalMethod.apply(handlerRef, [
                            event,
                            context,
                            callback,
                        ]);
                        if (options?.captureResponse ?? true) {
                            tracerRef.addResponseAsMetadata(result, process.env._HANDLER);
                        }
                    }
                    catch (error) {
                        tracerRef.addErrorAsMetadata(error);
                        throw error;
                    }
                    finally {
                        try {
                            subsegment?.close();
                        }
                        catch (error) {
                            console.warn(`Failed to close or serialize segment, ${subsegment?.name}. We are catching the error but data might be lost.`, error);
                        }
                    }
                    return result;
                });
            };
            return descriptor;
        };
    }
    captureMethod(options) {
        return (_target, propertyKey, descriptor) => {
            const originalMethod = descriptor.value;
            const tracerRef = this;
            descriptor.value = function (...args) {
                if (!tracerRef.isTracingEnabled()) {
                    return originalMethod.apply(this, [...args]);
                }
                const methodName = String(propertyKey);
                const subsegmentName = options?.subSegmentName
                    ? options.subSegmentName
                    : `### ${methodName}`;
                return tracerRef.provider.captureAsyncFunc(subsegmentName, async (subsegment) => {
                    let result;
                    try {
                        result = await originalMethod.apply(this, [...args]);
                        if (options?.captureResponse ?? true) {
                            tracerRef.addResponseAsMetadata(result, methodName);
                        }
                    }
                    catch (error) {
                        tracerRef.addErrorAsMetadata(error);
                        throw error;
                    }
                    finally {
                        try {
                            subsegment?.close();
                        }
                        catch (error) {
                            console.warn(`Failed to close or serialize segment, ${subsegment?.name}. We are catching the error but data might be lost.`, error);
                        }
                    }
                    return result;
                });
            };
            return descriptor;
        };
    }
    getRootXrayTraceId() {
        return this.envVarsService.getXrayTraceId();
    }
    getSegment() {
        if (!this.isTracingEnabled()) {
            return new aws_xray_sdk_core_1.Subsegment('## Dummy segment');
        }
        const segment = this.provider.getSegment();
        if (segment === undefined) {
            console.warn('Failed to get the current sub/segment from the context, this is likely because you are not using the Tracer in a Lambda function.');
        }
        return segment;
    }
    isTraceSampled() {
        if (!this.isTracingEnabled())
            return false;
        return this.envVarsService.getXrayTraceSampled();
    }
    isTracingEnabled() {
        return this.tracingEnabled;
    }
    putAnnotation(key, value) {
        if (!this.isTracingEnabled())
            return;
        this.provider.putAnnotation(key, value);
    }
    putMetadata(key, value, namespace) {
        if (!this.isTracingEnabled())
            return;
        this.provider.putMetadata(key, value, namespace || this.serviceName);
    }
    setSegment(segment) {
        if (!this.isTracingEnabled())
            return;
        return this.provider.setSegment(segment);
    }
    getCustomConfigService() {
        return this.customConfigService;
    }
    getEnvVarsService() {
        return this.envVarsService;
    }
    isAmplifyCli() {
        return (this.getEnvVarsService().getAwsExecutionEnv() ===
            'AWS_Lambda_amplify-mock');
    }
    isLambdaExecutionEnv() {
        return this.getEnvVarsService().getAwsExecutionEnv() !== '';
    }
    isLambdaSamCli() {
        return this.getEnvVarsService().getSamLocal() !== '';
    }
    setCaptureError() {
        const customConfigValue = this.getCustomConfigService()?.getTracingCaptureError();
        if (customConfigValue !== undefined &&
            customConfigValue.toLowerCase() === 'false') {
            this.captureError = false;
            return;
        }
        const envVarsValue = this.getEnvVarsService().getTracingCaptureError();
        if (envVarsValue.toLowerCase() === 'false') {
            this.captureError = false;
            return;
        }
    }
    setCaptureHTTPsRequests(enabled) {
        if (enabled !== undefined && !enabled) {
            this.captureHTTPsRequests = false;
            return;
        }
        const customConfigValue = this.getCustomConfigService()?.getCaptureHTTPsRequests();
        if (customConfigValue !== undefined &&
            customConfigValue.toLowerCase() === 'false') {
            this.captureHTTPsRequests = false;
            return;
        }
        const envVarsValue = this.getEnvVarsService().getCaptureHTTPsRequests();
        if (envVarsValue.toLowerCase() === 'false') {
            this.captureHTTPsRequests = false;
            return;
        }
    }
    setCaptureResponse() {
        const customConfigValue = this.getCustomConfigService()?.getTracingCaptureResponse();
        if (customConfigValue !== undefined &&
            customConfigValue.toLowerCase() === 'false') {
            this.captureResponse = false;
            return;
        }
        const envVarsValue = this.getEnvVarsService().getTracingCaptureResponse();
        if (envVarsValue.toLowerCase() === 'false') {
            this.captureResponse = false;
            return;
        }
    }
    setCustomConfigService(customConfigService) {
        this.customConfigService = customConfigService
            ? customConfigService
            : undefined;
    }
    setEnvVarsService() {
        this.envVarsService = new config_1.EnvironmentVariablesService();
    }
    setOptions(options) {
        const { enabled, serviceName, captureHTTPsRequests, customConfigService } = options;
        this.setEnvVarsService();
        this.setCustomConfigService(customConfigService);
        this.setTracingEnabled(enabled);
        this.setCaptureResponse();
        this.setCaptureError();
        this.setServiceName(serviceName);
        this.setCaptureHTTPsRequests(captureHTTPsRequests);
        return this;
    }
    setServiceName(serviceName) {
        if (serviceName !== undefined && this.isValidServiceName(serviceName)) {
            this.serviceName = serviceName;
            return;
        }
        const customConfigValue = this.getCustomConfigService()?.getServiceName();
        if (customConfigValue !== undefined &&
            this.isValidServiceName(customConfigValue)) {
            this.serviceName = customConfigValue;
            return;
        }
        const envVarsValue = this.getEnvVarsService().getServiceName();
        if (envVarsValue !== undefined && this.isValidServiceName(envVarsValue)) {
            this.serviceName = envVarsValue;
            return;
        }
        this.serviceName = this.getDefaultServiceName();
    }
    setTracingEnabled(enabled) {
        if (enabled !== undefined && !enabled) {
            this.tracingEnabled = enabled;
            return;
        }
        const customConfigValue = this.getCustomConfigService()?.getTracingEnabled();
        if (customConfigValue !== undefined &&
            customConfigValue.toLowerCase() === 'false') {
            this.tracingEnabled = false;
            return;
        }
        const envVarsValue = this.getEnvVarsService().getTracingEnabled();
        if (envVarsValue.toLowerCase() === 'false') {
            this.tracingEnabled = false;
            return;
        }
        if (this.isAmplifyCli() ||
            this.isLambdaSamCli() ||
            !this.isLambdaExecutionEnv()) {
            this.tracingEnabled = false;
        }
    }
}
exports.Tracer = Tracer;
